@@ -1,4 +1,4 @@
-# Copyright (c) 2022-2024 Mario S. Könz; License: MIT
+# Copyright (c) 2022-2025 Mario S. Könz; License: MIT
 import collections
 import dataclasses as dc
 import enum
@@ -15,8 +15,11 @@ from django.db.models.fields.files import FieldFile
 from ._create_django_model import InspectDataclass
 from ._decorator import BACKEND_LINKER
 from ._decorator import django_model
+from ._model_fields import iter_model_fields
+from ._model_fields import ModelFieldInfo
 from ._path_proxy import DjangoPath
 from ._protocols import T
+from ._util import streaming_file_cmp
 
 
 @dc.dataclass
@@ -28,9 +31,14 @@ class DjangoStore:
     ) -> "tuple[models.Model, bool]":
         kwgs, m2m, update_origin = self.dataclass_to_django_kwgs(dc_obj)
         identifying, defaults = self._split_off_pk(dc_obj, kwgs)
-        dj_obj, created = self.backend_manager(dc_obj).update_or_create(
-            **identifying, defaults=defaults
-        )
+        if identifying:
+            dj_obj, created = self.backend_manager(dc_obj).update_or_create(
+                **identifying, defaults=defaults
+            )
+        else:
+            dj_obj = self.backend_manager(dc_obj).create(**defaults)
+            created = True
+
         for key, fct in update_origin.items():
             try:
                 setattr(dc_obj, key, fct(getattr(dj_obj, key)))
@@ -79,14 +87,15 @@ class DjangoStore:
         kwgs = {}
         m2m = {}
 
-        to_process = {field.name for field in dc.fields(dc_obj)}
+        field_iterable = list(iter_model_fields(dc_obj.__class__))
+        to_process = {field.name for field in field_iterable}
         # sometimes we only want the fields necessary for
         # identifying the object of interest
         if pk_only:
             to_process = InspectDataclass(dc_obj).get_identifying_parameter()
         update_origin: dict[str, tp.Any] = {}
         weak_ref = self._get_weak_ref(dc_obj)
-        for field in dc.fields(dc_obj):
+        for field in field_iterable:
             key = field.name
             if key not in to_process:
                 continue
@@ -109,10 +118,10 @@ class DjangoStore:
 
         return kwgs, m2m, update_origin
 
-    def _process_path(
+    def _process_path(  # pylint: disable=too-many-return-statements,too-many-branches,too-many-locals
         self,
         val: tp.Any,
-        field: dc.Field[Path],
+        field: ModelFieldInfo,
         model: type[models.Model],
         update_origin: dict[str, tp.Any],
     ) -> tp.Any:
@@ -122,41 +131,71 @@ class DjangoStore:
         # pylint: disable=protected-access
         dj_field = model._meta.get_field(key)
         create_dj_path = self.create_dj_path(model, field)
+
+        def are_files_identical(prefix_name: str) -> bool:
+            incomming = val.open("rb")
+            existing = storage.open(prefix_name, "rb")
+            return streaming_file_cmp(incomming, existing)  # type: ignore
+
         if isinstance(val, DjangoPath):
             # the DjangoPath can be from another Model,
             # hence we strip the prefix and add it again
             name = val.name_wo_prefix()
             prefix_name = dj_field.generate_filename(None, name)  # type: ignore
             cross_model = prefix_name != val.as_posix()
-            if resave == "always" or (
-                cross_model and resave == "first" and not storage.exists(prefix_name)
-            ):
-                update_origin[key] = create_dj_path
 
+            def uploaded_file() -> UploadedFile:
+                update_origin[key] = create_dj_path
                 return UploadedFile(self._get_content(field, val), name)
-            if resave == "first":
+
+            if resave == "always":
+                return uploaded_file()
+            if resave in ["first", "on_change"]:
                 if not storage.exists(prefix_name):
+                    if cross_model:
+                        return uploaded_file()
                     raise FileNotFoundError(storage.path(prefix_name))
+
+                if resave == "on_change":
+                    # Note: we are not doing this, bc if its the same model, its already stored
+                    #       and it should not be different, since it came from the StorageBackend.
+                    #       In short: DjangoPaths come from the Backend, and dont hold new versions.
+                    # if not are_files_identical(prefix_name):
+                    #     return uploaded_file()
+                    pass
+
                 if cross_model:
                     return prefix_name
                 val._committed = True  # type: ignore
                 return val
+
             raise NotImplementedError(resave)
         if val.exists():
             prefix_name = dj_field.generate_filename(None, val.name)  # type: ignore
             update_origin[key] = create_dj_path
             exists = storage.exists(prefix_name)
+
             if resave == "first" and exists:
                 return prefix_name
-            if resave == "always" or (resave == "first" and not exists):
+            if resave == "on_change" and exists:
+                if are_files_identical(prefix_name):
+                    return prefix_name
+
+            if resave in ["always", "first", "on_change"]:
+                name = val.name
+                if resave == "on_change" and exists:
+                    # this is a very specific need bc of how django
+                    # handles the presents or absence of upload_to
+                    # on overwrites.
+                    name = storage.get_available_name(name)
                 return File(
                     self._get_content(field, val),
-                    val.name,
+                    name,
                 )
             raise NotImplementedError(resave)
         raise FileNotFoundError(val)
 
-    def _get_content(self, field: dc.Field[Path], val: Path) -> tp.Any:
+    def _get_content(self, field: ModelFieldInfo, val: Path) -> tp.Any:
         if val.is_dir():
             if field.metadata.get("allow_dir", False):
                 return val
@@ -166,7 +205,7 @@ class DjangoStore:
         return val.open("rb")
 
     def create_dj_path(
-        self, model: type[models.Model], dc_field: dc.Field[Path]
+        self, model: type[models.Model], dc_field: ModelFieldInfo
     ) -> tp.Callable[[FieldFile], DjangoPath]:
         # pylint: disable=protected-access
         dj_field = model._meta.get_field(dc_field.name)
@@ -197,7 +236,7 @@ class DjangoStore:
                 truekeys.add(key)
         return truekeys, subkeys
 
-    def django_to_dataclass(  # pylint: disable=too-many-locals
+    def django_to_dataclass(  # pylint: disable=too-many-locals,too-many-branches
         self,
         dj_obj: models.Model,
         defer: tp.Iterable[str] = tuple(),
@@ -207,27 +246,32 @@ class DjangoStore:
         obj_kwgs: dict[str, tp.Any] = {}
         true_defer, sub_defer = self._true_key_and_subkey(defer)
         true_only, sub_only = self._true_key_and_subkey(only, is_only=True)
-        for field in dc.fields(dataclass):
+        for field in iter_model_fields(dataclass):
             key = field.name
             if key in true_defer or (true_only and key not in true_only):
-                obj_kwgs[field.name] = None
+                obj_kwgs[key] = None
                 continue
             val = getattr(dj_obj, key)
             if type(val) in BACKEND_LINKER.backend_to_dc:
                 val = self.django_to_dataclass(
                     val,
-                    defer=sub_defer.get(field.name, set()),
-                    only=sub_only.get(field.name, set()),
+                    defer=sub_defer.get(key, set()),
+                    only=sub_only.get(key, set()),
                 )
 
-            field_type = field.type
+            field_type = field.annotation
             origin = tp.get_origin(field_type)
+
+            if origin == tp.Annotated:
+                field_type = tp.get_args(field_type)[0]
+                origin = tp.get_origin(field_type)
+
             if origin:
                 # pylint: disable=protected-access
                 dj_field = dj_obj._meta.get_field(key)
                 if origin == types.UnionType:
                     field_type, none_type = tp.get_args(field_type)
-                    if none_type != type(None):
+                    if none_type != type(None):  # pylint: disable=unidiomatic-typecheck
                         raise NotImplementedError(
                             "Union not supported yet, except for Optional"
                         )
@@ -239,19 +283,19 @@ class DjangoStore:
 
                 elif origin == set:
                     assert isinstance(dj_field, models.ManyToManyField)
-                    defer = sub_defer.get(field.name, set())
-                    only = sub_only.get(field.name, set())
+                    defer = sub_defer.get(key, set())
+                    only = sub_only.get(key, set())
                     query = val.defer(*defer).only(*only)
                     val = {self.django_to_dataclass(x, defer, only) for x in query}
                 else:
                     raise RuntimeError(f"field type {origin} not supported yet!")
 
-            if issubclass(field_type, enum.Enum):
+            if isinstance(field_type, type) and issubclass(field_type, enum.Enum):
                 val = field_type(val)
-            if issubclass(field_type, Path):
+            if isinstance(field_type, type) and issubclass(field_type, Path):
                 create_dj_path = self.create_dj_path(dj_obj.__class__, field)
                 val = create_dj_path(val)  # contains prefix
-            obj_kwgs[field.name] = val
+            obj_kwgs[key] = val
         return dataclass(**obj_kwgs)
 
     parse = django_to_dataclass
