@@ -1,4 +1,4 @@
-# Copyright (c) 2022-2024 Mario S. Könz; License: MIT
+# Copyright (c) 2022-2025 Mario S. Könz; License: MIT
 import collections
 import contextlib
 import io
@@ -18,9 +18,9 @@ from ._protocols import BackendStoreProtocol
 
 __all__ = ["set_store", "set_store_for_django", "ACTIVE_STORES", "set_file_backend"]
 
-ACTIVE_STORES: collections.OrderedDict[
-    str, BackendStoreProtocol
-] = collections.OrderedDict()
+ACTIVE_STORES: collections.OrderedDict[str, BackendStoreProtocol] = (
+    collections.OrderedDict()
+)
 
 
 def set_store(
@@ -38,6 +38,20 @@ def set_store(
                 USE_TZ=True,
             )
             ACTIVE_STORES[identifier] = DjangoStore(identifier)
+    elif flavor in ["postgres"]:
+        with django_setup(identifier) as databases:
+            assert identifier not in databases
+            databases[identifier] = dict(
+                NAME=kwgs.get("name", "postgres"),
+                USER=kwgs.get("user", "postgres"),
+                PASSWORD=kwgs.get("password", "postgres"),
+                HOST=kwgs["host"],
+                PORT=kwgs.get("port", "5432"),
+                ENGINE="django.db.backends.postgresql",
+                TIME_ZONE="UTC",
+                USE_TZ=True,
+            )
+            ACTIVE_STORES[identifier] = DjangoStore(identifier)
     else:
         raise NotImplementedError(flavor)
 
@@ -56,7 +70,6 @@ def django_setup(identifier: str) -> tp.Iterator[dict[str, tp.Any]]:
     databases = settings.DATABASES
     yield databases
     new_apps = set(BACKEND_LINKER.app_labels) - set(settings.INSTALLED_APPS)
-
     if new_apps:
         for app_label in BACKEND_LINKER.app_labels:
             if app_label in new_apps:
